@@ -1,17 +1,63 @@
 import copy
+from itertools import product
 import os
 from typing import Iterable
 import warnings
 
+import gym
 import numpy as np
 import pytest
 from pyvirtualdisplay import Display
 
-from plangym.core import PlanEnvironment
+import plangym
+from plangym.core import PlanEnv, PlangymEnv
 from plangym.vectorization.env import VectorizedEnvironment
 from plangym.videogames.env import LIFE_KEY
 
 
+def generate_test_cases(
+    names,
+    env_class,
+    n_workers_values=None,
+    render_modes=None,
+    obs_types=None,
+    custom_tests=None,
+) -> PlangymEnv:
+    custom_tests = custom_tests or []
+    n_workers_vals = [None] if n_workers_values is None else n_workers_values
+    names = [names] if isinstance(names, str) else names
+    available_render_modes = (
+        [None] if os.getenv("SKIP_RENDER", False) else env_class.AVAILABLE_RENDER_MODES
+    )
+    available_obs_types = (
+        [None] if os.getenv("SKIP_RENDER", False) else env_class.AVAILABLE_OBS_TYPES
+    )
+    render_modes = available_render_modes if render_modes is None else render_modes
+    obs_types = available_obs_types if obs_types is None else obs_types
+    for i, (n_workers, obs_type, render_mode) in enumerate(
+        product(
+            n_workers_vals,
+            obs_types,
+            render_modes,
+        ),
+    ):
+        name = names[i % len(names)]
+        if isinstance(name, tuple):
+            name = "-".join(name)
+
+        def _make_env():
+            return plangym.make(
+                name,
+                n_workers=n_workers,
+                obs_type=obs_type,
+                render_mode=render_mode,
+            )
+
+        yield _make_env
+    for custom_test in custom_tests:
+        yield custom_test
+
+
 @pytest.fixture(scope="class")
 def batch_size() -> int:
     return 10
@@ -28,7 +74,7 @@ def display():
 def step_tuple_test(env, obs, reward, terminal, info, dt=None):
     obs_is_array = isinstance(obs, np.ndarray)
     assert obs_is_array if env.OBS_IS_ARRAY else not obs_is_array
-    assert obs.shape == env.obs_shape
+    assert obs.shape == env.obs_shape, (obs.shape, env.obs_shape)
     assert float(reward) + 1 == float(reward) + 1
     assert isinstance(terminal, bool)
     assert isinstance(info, dict)
@@ -53,7 +99,7 @@ def step_batch_tuple_test(env, batch_size, observs, rewards, terminals, infos, d
         step_tuple_test(env=env, obs=obs, reward=reward, terminal=terminal, info=info, dt=dt)
 
 
-class TestPlanEnvironment:
+class TestPlanEnv:
 
     CLASS_ATTRIBUTES = ("OBS_IS_ARRAY", "STATE_IS_ARRAY", "SINGLETON")
     PROPERTIES = (
@@ -84,6 +130,17 @@ class TestPlanEnvironment:
     def test_name(self, env):
         assert isinstance(env.name, str)
 
+    def test_obs_shape(self, env):
+        assert hasattr(env, "obs_shape")
+        assert isinstance(env.obs_shape, tuple)
+        if env.obs_shape:
+            for val in env.obs_shape:
+                assert isinstance(val, int)
+        obs = env.reset(return_state=False)
+        assert obs.shape == env.obs_shape
+        obs, *_ = env.step(env.sample_action())
+        assert obs.shape == env.obs_shape
+
     def test_action_shape(self, env):
         assert hasattr(env, "action_shape")
         assert isinstance(env.action_shape, tuple)
@@ -91,15 +148,8 @@ class TestPlanEnvironment:
             for val in env.action_shape:
                 assert isinstance(val, int)
 
-    def test_obs_shape(self, env):
-        assert hasattr(env, "obs_shape")
-        assert isinstance(env.obs_shape, tuple)
-        if env.obs_shape:
-            for val in env.obs_shape:
-                assert isinstance(val, int)
-
     def test_unwrapped(self, env):
-        assert isinstance(env.unwrapped, PlanEnvironment)
+        assert isinstance(env.unwrapped, PlanEnv)
 
     @pytest.mark.skipif(os.getenv("SKIP_RENDER", False), reason="No display in CI.")
     @pytest.mark.parametrize("return_image", [True, False])
@@ -132,13 +182,13 @@ class TestPlanEnvironment:
     def test_set_state(self, env):
         env.reset()
         state = env.get_state()
-        env.set_state(state)
         env.step(env.sample_action())
+        env.set_state(state)
         if env.STATE_IS_ARRAY:
             env_state = env.get_state()
             assert state.shape == env_state.shape
-            if state.dtype != object:
-                assert not (state == env_state).all(), (state, env.get_state())
+            if state.dtype != object and not env.SINGLETON:
+                assert (state == env_state).all(), (state, env.get_state())
 
     def test_reset(self, env):
         _ = env.reset(return_state=False)
@@ -147,14 +197,10 @@ class TestPlanEnvironment:
         obs_is_array = isinstance(obs, np.ndarray)
         assert state_is_array if env.STATE_IS_ARRAY else not state_is_array
         assert obs_is_array if env.OBS_IS_ARRAY else not obs_is_array
-        # assert (
-        #    env.obs_shape == obs.shape
-        # ), f"env.obs_shape {tuple(env.obs_shape)}, obs: {obs.shape}"
 
-    @pytest.mark.parametrize("dt", [1, 3])
     @pytest.mark.parametrize("state", [None, True])
     @pytest.mark.parametrize("return_state", [None, True, False])
-    def test_step(self, env, state, dt, return_state):
+    def test_step(self, env, state, return_state, dt=1):
         _state, *_ = env.reset(return_state=True)
         if state is not None:
             state = _state
@@ -173,18 +219,19 @@ class TestPlanEnvironment:
             if state_is_array:
                 assert _state.shape == new_state.shape
             if not env.SINGLETON:
-                assert (
-                    new_state == env.get_state()
-                ).all(), f"original: {state} env: {env.get_state()}"
+                curr_state = env.get_state()
+                assert (new_state == curr_state).all(), (
+                    f"original: {new_state[new_state!= curr_state]} "
+                    f"env: {curr_state[new_state!= curr_state]}"
+                )
         else:
             assert len(new_state) == 0
         step_tuple_test(env, obs, reward, terminal, info, dt=dt)
 
-    @pytest.mark.parametrize("dt", [1, 3, "array"])
     @pytest.mark.parametrize("states", [None, True, "None_list"])
     @pytest.mark.parametrize("return_state", [None, True, False])
-    def test_step_batch(self, env, dt, states, return_state, batch_size):
-        dt = dt if dt != "array" else np.random.randint(1, 4, batch_size).astype(int)
+    def test_step_batch(self, env, states, return_state, batch_size):
+        dt = 1
         state, _ = env.reset()
         if states == "None_list":
             states = [None] * batch_size
@@ -223,6 +270,38 @@ class TestPlanEnvironment:
             dt=dt,
         )
 
+    def test_step_dt_values(self, env, dt=3, return_state=None):
+        state = None
+        _state, *_ = env.reset(return_state=True)
+        action = env.sample_action()
+
+        data = env.step(action, dt=dt, state=state, return_state=return_state)
+        *new_state, obs, reward, terminal, info = data
+        assert isinstance(data, tuple)
+        assert len(new_state) == 0
+        step_tuple_test(env, obs, reward, terminal, info, dt=dt)
+
+    @pytest.mark.parametrize("dt", [3, "array"])
+    def test_step_batch_dt_values(self, env, dt, batch_size, states=None, return_state=None):
+        dt = dt if dt != "array" else np.random.randint(1, 4, batch_size).astype(int)
+        state, _ = env.reset()
+        actions = [env.sample_action() for _ in range(batch_size)]
+
+        data = env.step_batch(actions, dt=dt, states=states, return_state=return_state)
+        *new_states, observs, rewards, terminals, infos = data
+        assert isinstance(data, tuple)
+        assert len(new_states) == 0, (len(new_states), return_state)
+
+        step_batch_tuple_test(
+            env=env,
+            batch_size=batch_size,
+            observs=observs,
+            rewards=rewards,
+            terminals=terminals,
+            infos=infos,
+            dt=dt,
+        )
+
     def test_clone_and_close(self, env):
         if not env.SINGLETON:
             clone = env.clone()
@@ -243,8 +322,8 @@ class TestPlanEnvironment:
             assert len(img.shape) == 2 or len(img.shape) == 3
 
 
-class TestPlangymEnv(TestPlanEnvironment):
-    CLASS_ATTRIBUTES = ("AVAILABLE_OBS_TYPE", "DEFAULT_OBS_TYPE")
+class TestPlangymEnv:
+    CLASS_ATTRIBUTES = ("AVAILABLE_OBS_TYPES", "DEFAULT_OBS_TYPE")
     PROPERTIES = (
         "gym_env",
         "obs_shape",
@@ -272,6 +351,44 @@ class TestPlangymEnv(TestPlanEnvironment):
         for name in self.PROPERTIES:
             assert hasattr(env, name), f"Env {env.name} does not have property {name}"
 
+    def test_obs_type(self, env):
+        assert isinstance(env.obs_type, str)
+        assert env.obs_type in env.AVAILABLE_OBS_TYPES
+        assert env.DEFAULT_OBS_TYPE in env.AVAILABLE_OBS_TYPES, (
+            str(env.DEFAULT_OBS_TYPE),
+            env.AVAILABLE_OBS_TYPES,
+        )
+
+    def test_obvervation_space(self, env):
+        assert hasattr(env, "observation_space")
+        if env.observation_space is None:
+            env.setup()
+        assert isinstance(env.observation_space, gym.Space), (
+            env.observation_space,
+            env.DEFAULT_OBS_TYPE,
+        )
+        assert env.observation_space.shape == env.obs_shape
+        if env.observation_space.shape:
+            assert env.observation_space.shape == env.reset(return_state=False).shape
+
+    def test_action_space(self, env):
+        assert hasattr(env, "action_space")
+        if env.action_space is None:
+            env.setup()
+        assert isinstance(env.action_space, gym.Space)
+        assert env.action_space.shape == env.action_shape
+        if env.action_space.shape:
+            assert env.action_space.shape == env.sample_action().shape
+
+    def test_gym_env(self, env):
+        assert hasattr(env.gym_env, "reset")
+        assert hasattr(env.gym_env, "step")
+        if not isinstance(env, VectorizedEnvironment) and not env.SINGLETON:
+            env.close()
+
+    def test_reward_range(self, env):
+        env.reward_range
+
     @pytest.mark.parametrize("delay_setup", [True, False])
     def test_delay_setup(self, env, delay_setup):
         if env.SINGLETON or isinstance(env, VectorizedEnvironment):
@@ -280,43 +397,19 @@ class TestPlangymEnv(TestPlanEnvironment):
         assert new_env._gym_env is None if delay_setup else new_env._gym_env is not None
         assert env.gym_env is not None
 
-    def test_action_space(self, env):
-        assert hasattr(env, "action_space")
-        # Singleton / delayed envs may not be properly initialized. In that case test separately
-        if env.sample_action() is not None:
-            action = env.action_space.sample()
-            assert action is not None
-
-    def test_api_kwargs(self, env):
-        if isinstance(env, VectorizedEnvironment):
-            return
-        cls = env.__class__
-        kwargs = {
-            "name": env.name,
-            "frameskip": 1,
-            "autoreset": True,
-            "delay_setup": True,
-            "wrappers": None,
-        }
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            cls(**kwargs)
-
-    def test_obs_space(self, env):
-        assert hasattr(env, "observation_space")
-
-    def test_attributes(self, env):
-        assert hasattr(env, "reward_range")
+    def test_has_metadata(self, env):
         assert hasattr(env, "metadata")
-        # assert hasattr(env, "episodic_life")
-        assert hasattr(env, "gym_env")
 
-    def __test_get_lifes_from_info(self, env):
-        info = {"lifes": 3}
-        lifes = env.get_lifes_from_info(info)
-        assert lifes == 3
-        lifes = env.get_lifes_from_info({})
-        assert lifes == -1
+    def test_render_mode(self, env):
+        assert hasattr(env, "render_mode")
+        if env.render_mode is not None:
+            assert isinstance(env.render_mode, str)
+        assert env.render_mode in env.AVAILABLE_RENDER_MODES
+
+    def test_remove_time_limit(self, env):
+        assert isinstance(env.remove_time_limit, bool)
+        if env.remove_time_limit and not env._wrappers:
+            assert "TimeLimit" not in str(env.gym_env), env.gym_env
 
     def test_seed(self, env):
         env.seed()
@@ -333,10 +426,22 @@ class TestPlangymEnv(TestPlanEnvironment):
             warnings.simplefilter("ignore")
             env.render()
 
-    # TODO: add after finishing wrappers
-    def _test_wrap_environment(self, env):
-        wrappers = []
-        env.apply_wrappers(wrappers)
+    def test_wrap_environment(self, env):
+        if isinstance(env, VectorizedEnvironment):
+            return
+        from gym.wrappers.transform_reward import TransformReward
 
-    def _test_apply_wrappers(self, env):
-        pass
+        wrappers = [(TransformReward, {"f": lambda x: x})]
+        env.apply_wrappers(wrappers)
+        assert isinstance(env.gym_env, TransformReward)
+        env._gym_env = env.gym_env.env
+
+        wrappers = [(TransformReward, [lambda x: x])]
+        env.apply_wrappers(wrappers)
+        assert isinstance(env.gym_env, TransformReward)
+        env._gym_env = env.gym_env.env
+
+        wrappers = [(TransformReward, lambda x: x)]
+        env.apply_wrappers(wrappers)
+        assert isinstance(env.gym_env, TransformReward)
+        env._gym_env = env.gym_env.env
