@@ -14,7 +14,7 @@ from plangym.utils import process_frame, remove_time_limit
 wrap_callable = Union[Callable[[], gym.Wrapper], Tuple[Callable[..., gym.Wrapper], Dict[str, Any]]]
 
 
-class PlanEnvironment(ABC):
+class PlanEnv(ABC):
     """Inherit from this class to adapt environments to different problems."""
 
     STATE_IS_ARRAY = True
@@ -68,7 +68,22 @@ class PlanEnvironment(ABC):
 
     # Public API -----------------------------------------------------------------------------
     @property
-    def unwrapped(self) -> "PlanEnvironment":
+    def name(self) -> str:
+        """Return is the name of the environment."""
+        return self._name
+
+    @property
+    def obs_shape(self) -> Tuple[int]:
+        """Tuple containing the shape of the observations returned by the Environment."""
+        raise NotImplementedError()
+
+    @property
+    def action_shape(self) -> Tuple[int]:
+        """Tuple containing the shape of the actions applied to the Environment."""
+        raise NotImplementedError()
+
+    @property
+    def unwrapped(self) -> "PlanEnv":
         """
         Completely unwrap this Environment.
 
@@ -88,21 +103,6 @@ class PlanEnvironment(ABC):
         """
         return self._return_image
 
-    @property
-    def name(self) -> str:
-        """Return is the name of the environment."""
-        return self._name
-
-    @property
-    def obs_shape(self) -> Tuple[int]:
-        """Tuple containing the shape of the observations returned by the Environment."""
-        raise NotImplementedError()
-
-    @property
-    def action_shape(self) -> Tuple[int]:
-        """Tuple containing the shape of the actions applied to the Environment."""
-        raise NotImplementedError()
-
     def get_image(self) -> Union[None, np.ndarray]:
         """
         Return a numpy array containing the rendered view of the environment.
@@ -212,7 +212,7 @@ class PlanEnvironment(ABC):
         ]
         return tuple(list(x) for x in zip(*data))
 
-    def clone(self, **kwargs) -> "PlanEnvironment":
+    def clone(self, **kwargs) -> "PlanEnv":
         """Return a copy of the environment."""
         clone_kwargs = dict(
             name=self.name,
@@ -223,7 +223,7 @@ class PlanEnvironment(ABC):
         clone_kwargs.update(kwargs)
         return self.__class__(**clone_kwargs)
 
-    def sample_action(self):
+    def sample_action(self):  # pragma: no cover
         """
         Return a valid action that can be used to step the Environment.
 
@@ -450,10 +450,11 @@ class PlanEnvironment(ABC):
         raise NotImplementedError()
 
 
-class PlangymEnv(PlanEnvironment):
+class PlangymEnv(PlanEnv):
     """Base class for implementing OpenAI ``gym`` environments in ``plangym``."""
 
-    AVAILABLE_OBS_TYPE = {"coords", "rgb", "grayscale"}
+    AVAILABLE_RENDER_MODES = {"human", "rgb_array", None}
+    AVAILABLE_OBS_TYPES = {"coords", "rgb", "grayscale"}
     DEFAULT_OBS_TYPE = "coords"
 
     def __init__(
@@ -494,9 +495,9 @@ class PlangymEnv(PlanEnvironment):
         self._obs_space = None
         self._action_space = None
         if obs_type is not None:
-            assert obs_type in self.AVAILABLE_OBS_TYPE, (
+            assert obs_type in self.AVAILABLE_OBS_TYPES, (
                 f"obs_type {obs_type} is not accepted. Available "
-                f"values are: {self.AVAILABLE_OBS_TYPE}"
+                f"values are: {self.AVAILABLE_OBS_TYPES}"
             )
         self._obs_type = obs_type or self.DEFAULT_OBS_TYPE
         super(PlangymEnv, self).__init__(
@@ -517,10 +518,7 @@ class PlangymEnv(PlanEnvironment):
     @property
     def obs_shape(self) -> Tuple[int, ...]:
         """Tuple containing the shape of the observations returned by the Environment."""
-        try:
-            return self.observation_space.shape
-        except AttributeError:
-            return ()
+        return self.observation_space.shape
 
     @property
     def obs_type(self) -> str:
@@ -535,10 +533,7 @@ class PlangymEnv(PlanEnvironment):
     @property
     def action_shape(self) -> Tuple[int, ...]:
         """Tuple containing the shape of the actions applied to the Environment."""
-        try:
-            return self.action_space.shape
-        except AttributeError:
-            return ()
+        return self.action_space.shape
 
     @property
     def action_space(self) -> Space:
@@ -586,6 +581,8 @@ class PlangymEnv(PlanEnvironment):
             self._init_obs_space_grayscale()
         elif self.obs_type == "coords":
             self._init_obs_space_coords()
+        if self.observation_space is None:
+            self._obs_space = self.gym_env.observation_space
 
     def _init_action_space(self):
         self._action_space = self.gym_env.action_space
@@ -606,8 +603,8 @@ class PlangymEnv(PlanEnvironment):
             self._gym_env = GrayScaleObservation(self._gym_env)
             self._obs_space = self._gym_env.observation_space
         else:
-            img_shape = self.get_image().shape
-            self._obs_space = Box(low=0, high=255, dtype=numpy.uint8, shape=img_shape)
+            shape = self.get_image().shape
+            self._obs_space = Box(low=0, high=255, dtype=numpy.uint8, shape=(shape[0], shape[1]))
 
     def _init_obs_space_coords(self):
         if self.DEFAULT_OBS_TYPE == "coords":
@@ -654,10 +651,7 @@ class PlangymEnv(PlanEnvironment):
         """Return a valid action that can be used to step the Environment chosen at random."""
         if hasattr(self.action_space, "sample"):
             return self.action_space.sample()
-        try:
-            return self.gym_env.action_space.sample()
-        except AttributeError:
-            pass
+        return self.gym_env.action_space.sample()
 
     def clone(self, **kwargs) -> "PlangymEnv":
         """Return a copy of the environment."""
@@ -675,6 +669,7 @@ class PlangymEnv(PlanEnvironment):
         """Close the underlying :class:`gym.Env`."""
         if hasattr(self, "_gym_env") and hasattr(self._gym_env, "close"):
             return self._gym_env.close()
+        self._gym_env = None
 
     def init_gym_env(self) -> gym.Env:
         """Initialize the :class:`gym.Env`` instance that the current class is wrapping."""
@@ -692,7 +687,12 @@ class PlangymEnv(PlanEnvironment):
         for item in wrappers:
             if isinstance(item, tuple):
                 wrapper, kwargs = item
-                self.wrap(wrapper, **kwargs)
+                if isinstance(kwargs, dict):
+                    self.wrap(wrapper, **kwargs)
+                elif isinstance(kwargs, (list, tuple)):
+                    self.wrap(wrapper, *kwargs)
+                else:
+                    self.wrap(wrapper, kwargs)
             else:
                 self.wrap(item)
 
