@@ -6,7 +6,7 @@ import traceback
 
 import numpy
 
-from plangym.core import PlanEnvironment
+from plangym.core import PlanEnv
 from plangym.vectorization.env import VectorizedEnvironment
 
 
@@ -225,7 +225,7 @@ class ExternalProcess:
                     if not conn.poll(0.1):
                         continue
                     message, payload = conn.recv()
-                except (EOFError, KeyboardInterrupt):
+                except (EOFError, KeyboardInterrupt):  # pragma: no cover
                     break
                 if message == self._ACCESS:
                     name = payload
@@ -467,7 +467,7 @@ class ParallelEnvironment(VectorizedEnvironment):
         # Initialize local copy last to tolerate singletons better
         super(ParallelEnvironment, self).setup()
 
-    def clone(self, **kwargs) -> "PlanEnvironment":
+    def clone(self, **kwargs) -> "PlanEnv":
         """Return a copy of the environment."""
         default_kwargs = dict(blocking=self.blocking)
         default_kwargs.update(kwargs)
